//Note to self:
//I should probably stop you flying above the limit.
//Probably a lot of bound checking for huge scores etc to stop them wrapping.
//I should probably make the player sprite left aligned so it can reach the left edge.

//Add rocks and unbreakables (metal?)
//Game over screen.
//Should probably move lookup tables to program memory. (from ram)
//We really need:

//Saving money etc.
//Game ending.
//Menu graphics.

#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <uzebox.h>
//for memmove
//#include <string.h>

#include "data/image.inc"

#include "data/midi.c"
#include "data/game1.c"
#include "data/patches.c"

#define SCREEN_WIDTH 240
#define SCREEN_HEIGHT 224
#define TILES_ON_X (SCREEN_WIDTH/8)
#define TILES_ON_Y (SCREEN_HEIGHT/8)
//30 and 28.
//A pixel is a byte. 8*8 = 64 bytes in an image
#define BYTES_IN_TILE 64

//Be careful changing this. It probably won't work.
#define OVERLAY_ROWS 2

#define TILE_WITH_MENU_ARROW 152

#define TILE_ENERGY_LEFT 153
#define TILE_ENERGY_RIGHT 154
#define TILE_ENERGY_BAR_FULL 159

//Note, an x cannot be larger than 255. So we have this define.
//This should take them offscreen (I hope)
#define SCORE_OFFSCREEN_ADD 0
#define TILE_WITH_SCORE_0 ('0'-32)
#define TILE_WITH_MONEY_SIGN (TILE_WITH_SCORE_0+12)

#define FIRST_SPRITE_FOR_SCORE 4
#define LAST_SPRITE_FOR_SCORE 9
#define FIRST_SPRITE_FOR_FUEL 10
#define LAST_SPRITE_FOR_FUEL 12


//player image sprites
#define TILE_PLAYER_TOP_LEFT 96
#define TILE_PLAYER_TOP_RIGHT (TILE_PLAYER_TOP_LEFT+1)
#define TILE_PLAYER_BOTTOM_LEFT (TILE_PLAYER_TOP_LEFT+2)
#define TILE_PLAYER_BOTTOM_RIGHT (TILE_PLAYER_TOP_LEFT+3)

#define TILE_PLAYER_TOP_LEFT_FIRE (TILE_PLAYER_BOTTOM_RIGHT+1)
#define TILE_PLAYER_TOP_RIGHT_FIRE (TILE_PLAYER_TOP_LEFT_FIRE+1)
#define TILE_PLAYER_BOTTOM_LEFT_FIRE (TILE_PLAYER_TOP_LEFT_FIRE+2)
#define TILE_PLAYER_BOTTOM_RIGHT_FIRE (TILE_PLAYER_TOP_LEFT_FIRE+3)

#define TILE_PLAYER_WALK_TOP_LEFT 144
#define TILE_PLAYER_WALK_TOP_RIGHT (TILE_PLAYER_WALK_TOP_LEFT+1)
#define TILE_PLAYER_WALK_BOTTOM_LEFT (TILE_PLAYER_WALK_TOP_LEFT+2)
#define TILE_PLAYER_WALK_BOTTOM_RIGHT (TILE_PLAYER_WALK_TOP_LEFT+3)

#define TILE_PLAYER_DIG_TOP_LEFT 148
#define TILE_PLAYER_DIG_TOP_RIGHT (TILE_PLAYER_DIG_TOP_LEFT+1)
#define TILE_PLAYER_DIG_BOTTOM_LEFT (TILE_PLAYER_DIG_TOP_LEFT+2)
#define TILE_PLAYER_DIG_BOTTOM_RIGHT (TILE_PLAYER_DIG_TOP_LEFT+3)



#define TILE_BLOCK 104

#define TILE_ROCK (TILE_BLOCK+(4*5))

unsigned char playerX = 50;
unsigned char playerY = 80;

unsigned int worldSeed = 0;

#define scrollYOffset (Screen.scrollY)

#define BLACK_TILE 5

#define PLAYER_LEFT_SIDE (playerX)
#define PLAYER_RIGHT_SIDE (playerX+15)
#define PLAYER_BOTTOM_SIDE (playerY+15)
#define PLAYER_TOP_SIDE (playerY)

#define PLAYER_TOP (playerY)

#define GET_TILE_AT_X_Y(x,y,scrollYOffset) (GetTile(((x)/8), (((y)+(scrollYOffset))/8)))

//unsigned char playerSpeed = 2;
#define playerSpeed 1


unsigned char animationSpeed = 0;
unsigned char animationSpeedHalf = 1;


//The higher the blocks shift the more common at start.
//REALLY WANTED THESE IN FLASH BUT IT SEEMS THIS IS FASTER FOR NOW. PROGMEM
const unsigned char lookupShiftForType[] ={ 99, 99, 99, 1, //For normal tile it doesn't matter as they'll always be normal
                                            99, 10,  6,  9,
                                             8,  4,  6,  3,
                                             2,  8,  5,  2};
//We give the tiles to be looked up via the type
//This is also the distribution aproximately when super low
#define BLOCK(iii) (TILE_BLOCK+((iii)*4))
const unsigned char lookupForTileNum[] ={BLOCK(0), BLOCK(0), BLOCK(0), BLOCK(4),
                                         BLOCK(0), BLOCK(1), BLOCK(1), BLOCK(5),
                                         BLOCK(2), BLOCK(2), BLOCK(3), BLOCK(3),
                                         BLOCK(4), BLOCK(5), BLOCK(5), BLOCK(5)};
//The money values below are given in 100s. First is BLOCK_TILE and then onwards.
const unsigned char lookupForMoneyForTileByOrderInImage[] =
                            {0, 1, 5, 20, 100, 0, 0, 0};

unsigned char jetpackLevel = 0;
unsigned char energyLevel = 0;
unsigned char digSpeedLevel = 0;


unsigned char jetpackSpeed = 2; //Must be at least 2 for gravity.
unsigned char maxFlySpeed = 25; //DO SOMETHING WITH THIS (AND DO FALL SPEED)

#define TILE_BLUE_0 160
#define TILE_RED_0 (TILE_BLUE_0+10)
#define TILE_BLUE_DOLLAR (TILE_RED_0+10)
#define TILE_RED_DOLLAR (TILE_BLUE_DOLLAR+1)
#define TILE_BLUE_ARROW 182
#define TILE_LV 183


#define MAGIC_TILE_FOR_FUEL 30
#define MAGIC_TILE_FOR_SHOP 40
#define MAGIC_TILE_FOR_SAVE 50
#define NUMBER_OF_LINES_JUST_ABOVE_GROUND 6

#define TOP_IMG(x) (184+((x)*4)-4)
const unsigned char tilesJustAboveGround[NUMBER_OF_LINES_JUST_ABOVE_GROUND][28/2] = {
                                {01,  01,  01,  01,  00,  00,  00,  00,  00,  00,  01,  01,  01,  00},
                                {TOP_IMG(8),  TOP_IMG(9),  TOP_IMG(10),  TOP_IMG(11),  00,  00,  00,  00,  00,  00,  00,  00,  00,  00},
                                {TOP_IMG(4), TOP_IMG(5),  TOP_IMG(6),  TOP_IMG(7),  00,  00,  00,  00,  00,  00,  TOP_IMG(17),  TOP_IMG(16),  TOP_IMG(15),  00},
                                {00,  TOP_IMG(1),  TOP_IMG(2),  TOP_IMG(3),  00,  00,  00,  00,  00,  00,  TOP_IMG(14),  TOP_IMG(13),  TOP_IMG(12),  00},
                                {TILE_BLOCK, TILE_ROCK, TILE_ROCK, TILE_ROCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK,  TILE_ROCK,  TILE_ROCK,  TILE_ROCK},
                                {TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK, TILE_BLOCK,  TILE_BLOCK}
                                };


//I'm pretty sure I don't need to make a global variable be set to 0?
//the var below is a bool by the way
char playerIsGoingUp = 0;
unsigned char playerVSpeed = 0x10;
//Start 500 down as unsigned are easier to deal with later
unsigned int topOfScreenBlocksDown = 500;

//Pretty sure that as it's global it'll be 0s
//So x=(28/2)/8 and y is the depth we want. Too bad I can't bit pack that nicely though
//#define DEPTH 420
//Could really go as far as 800 but the game gets pretty boring then.
#define DEPTH 350
#define TILES_IN_A_LINE (28/2)
#define BOOLS_NEEDED_TO_STORE_MAP (TILES_IN_A_LINE*DEPTH)
unsigned char tunneledArray[(BOOLS_NEEDED_TO_STORE_MAP+7)/8];

unsigned char nextRowCache[(28/2)];

unsigned int playerScore = 0;
unsigned int playerHoldingScore = 0;

unsigned int playerFuel = (255 << 8) + 255;

//This is going to end up overflowing which isn't really
//allowed in C but hopefully I'll get away with it.
unsigned int frameCount = 0;

unsigned char idleDrainEnergy = 12;
unsigned char flyOrMineDrainEnergy = 64;


void SetValuesFromLevels() {
  //if (digSpeedLevel == 0) {
        //default so nothing
    /*} else*/ if (digSpeedLevel == 1) {
        animationSpeed = 1;
        animationSpeedHalf = 0;
    } else if (digSpeedLevel == 2) {
        animationSpeed = 1;
        animationSpeedHalf = 1;
    } else if (digSpeedLevel == 3) {
        animationSpeed = 2;
        animationSpeedHalf = 0;
        //This is max.
    }
    
    
    //if (energyLevel == 0) {
        //default so nothing
    /*} else*/ if (energyLevel == 1) {
        idleDrainEnergy = 6;
        flyOrMineDrainEnergy = 32;
    } else if (energyLevel == 2) {
        //max
        idleDrainEnergy = 3;
        flyOrMineDrainEnergy = 16;
    }
    
    //if (jetpackLevel == 0) {
        //default so nothing 30
    /*} else*/ if (jetpackLevel == 1) {
        //jetpackSpeed = 2; //default
        maxFlySpeed = 35;
    } else if (jetpackLevel == 2) {
        //max
        jetpackSpeed = 3;
        flyOrMineDrainEnergy = 48;
    }
    
}

/*write may return an error or 0 on success*/

void writeSave() {
    struct EepromBlockStruct block;
    block.id = 160;
    unsigned int index = 0;
    unsigned char bytesIntoBlock = 2+2+3;
    block.data[0] = (worldSeed >> 8) & 0xFF;
    block.data[1] = worldSeed & 0xFF;
    
    block.data[2] = (playerScore >> 8) & 0xFF;
    block.data[3] = playerScore & 0xFF;
    
    
    //unsigned char jetpackLevel = 0;
//unsigned char energyLevel = 0;
//unsigned char digSpeedLevel = 0;
    
    block.data[4] = jetpackLevel;
    block.data[5] = energyLevel;
    block.data[6] = digSpeedLevel;
    
    while (1) {
        unsigned char data = tunneledArray[index];
        block.data[bytesIntoBlock] = data;
        
        bytesIntoBlock++;
        if (bytesIntoBlock >= 30) {
            EepromWriteBlock(&block);
            block.id = block.id + 1;
            bytesIntoBlock = 0;
        }
            
        index++;
        if (index >= ((BOOLS_NEEDED_TO_STORE_MAP+7)/8) ) {
	    if (bytesIntoBlock > 0) {
                block.id = block.id + 1;
                bytesIntoBlock = 0;
            }
            
            break;
        }
    }
    //done.
    //MAKE ALL FUNCTIONS TAKE VOID AS ARGUMENT.
    //I SHOULD CHECK FOR ERRORS IN THE WRITES!!!
    //I should probably store the total size at start or something.
    //PrintInt(6, 6, block.id, 1);
    
}

/*
 * Reads a data block in the specified structure.
 *
 * Returns: 
 *  0x00 = Success
 * 	0x01 = EEPROM_ERROR_INVALID_BLOCK
 *	0x02 = EEPROM_ERROR_FULL
 *	0x03 = EEPROM_ERROR_BLOCK_NOT_FOUND
 *	0x04 = EEPROM_ERROR_NOT_FORMATTED
 */

void loadSave() {
    unsigned int bytesLeftInMap = (BOOLS_NEEDED_TO_STORE_MAP+7)/8;
    struct EepromBlockStruct ebs;
    unsigned char id = 160;
    unsigned int indexInArray = 0;
    
    ebs.id = id;
    EepromReadBlock(id, &ebs);
    worldSeed = ebs.data[1];
    worldSeed += ebs.data[0] << 8;
    
    playerScore =  ebs.data[3];
    playerScore += ebs.data[2] << 8;

    //unsigned char jetpackLevel = 0;
//unsigned char energyLevel = 0;
//unsigned char digSpeedLevel = 0;
    
    jetpackLevel = ebs.data[4];
    energyLevel = ebs.data[5];
    digSpeedLevel = ebs.data[6];
    
    SetValuesFromLevels();
    
    unsigned int bytesIntoBlock = 2+2+3;

    while (bytesLeftInMap > 0) {
        ebs.id = id; //I'm pretty sure setting this is pointless.
        EepromReadBlock(id, &ebs);
	tunneledArray[indexInArray] = ebs.data[bytesIntoBlock];
	

	//Could do some fancy stuff with modulus here but it doesn't seem
	//worth it. It might even be slower.
	indexInArray++;
	bytesIntoBlock++;
	if (bytesIntoBlock >= 30) {
	  bytesIntoBlock = 0;
	  id++;
	}
	if (indexInArray >= (BOOLS_NEEDED_TO_STORE_MAP+7)/8) {
	  break;
	}
    }
    //Could sure do with error checking too.
    //No idea what to do if there's an error though.
    //I guess I should just print it and let the user deal with it?
    
}

void draw_player() {
    //If it's not fliped
    if (sprites[0].flags == SPRITE_BANK0) {
        //They are facing right
        //Top left
        sprites[0].x = playerX;
        sprites[0].y = playerY;
        //top right
        sprites[1].x = playerX+8;
        sprites[1].y = playerY;
        //bottom left
        sprites[2].x = playerX;
        sprites[2].y = playerY+8;
        //bottom right
        sprites[3].x = playerX+8;
        sprites[3].y = playerY+8;
    } else {
        //We need to swap our sprites around to look correct (they're also flipped on x)
        sprites[0].x = playerX+8;
        sprites[0].y = playerY;
        
        sprites[1].x = playerX;
        sprites[1].y = playerY;
        
        //top left with original top right sprite
        sprites[1].x = playerX;
        sprites[1].y = playerY;
        //bottom right with bottom left sprite
        sprites[2].x = playerX+8;
        sprites[2].y = playerY+8;
        //bottom left with bottom right sprite
        sprites[3].x = playerX;
        sprites[3].y = playerY+8;
    }
}

void draw_hold_score() {
    if (playerHoldingScore != 0) {
        SetTile(26, 1, TILE_RED_0);
        SetTile(27, 1, TILE_RED_0);
        
        unsigned int temp = playerHoldingScore;
        unsigned char xxx = 25;
        do {
            unsigned char digit = temp % 10;
            SetTile(xxx, 1, TILE_RED_0 + digit);
            temp /= 10;
            xxx--;
        } while (temp > 0);
        SetTile(xxx, 1, TILE_RED_DOLLAR);
        
    } else {
        Fill(20, 1, 8, 1, BLACK_TILE);
    }
}

void draw_score() {
    //playerScore
    /*unsigned int temp = playerScore;
    if (temp > 9999) {
        temp = 9999;
    }
    unsigned char currentSprite = FIRST_SPRITE_FOR_SCORE+1;
    for (currentSprite = FIRST_SPRITE_FOR_SCORE+1; currentSprite < LAST_SPRITE_FOR_SCORE; currentSprite++) {
        unsigned char digit = temp % 10;
        sprites[currentSprite].tileIndex = TILE_WITH_SCORE_0 + digit;
        temp /= 10;
    }*/
    PrintInt(25, 0, playerScore, 1);
    //PrintInt(25, 1, playerHoldingScore, 1);
    draw_hold_score();
}

void draw_fuel() {
    /*unsigned char temp = playerFuel >> 8;
    unsigned char currentSprite = FIRST_SPRITE_FOR_FUEL;
    for (currentSprite = FIRST_SPRITE_FOR_FUEL; currentSprite <= LAST_SPRITE_FOR_FUEL; currentSprite++) {
        unsigned char digit = temp % 10;
        sprites[currentSprite].tileIndex = TILE_WITH_SCORE_0 + digit;
        temp /= 10;
    }*/
    //TILE_ENERGY_BAR_FULL
    //the fuel var is 16 bits long.
    //We have 8 blocks to draw so we need the last 3 bits.
    unsigned char fullBlocks = playerFuel >> 13;
    Fill(0, 1, fullBlocks, 1, TILE_ENERGY_BAR_FULL);
    //The next 2 bits show the part state.
    unsigned char partBlocks = (playerFuel >> 11) & 3;

    SetTile(fullBlocks, 1, TILE_ENERGY_BAR_FULL-3+partBlocks);
    
    if (playerFuel < (24 << 8) && frameCount & (1 << 3)) {
        SetTile(fullBlocks, 1, BLACK_TILE);
    }
    SetTile(fullBlocks+1, 1, BLACK_TILE);
    
    /*unsigned char temp = playerFuel >> 8;
    PrintBinaryByte(5, 20, temp);
    temp = playerFuel & 0xFF;
    PrintBinaryByte(5, 21, temp);*/
    
    //PrintInt(20, 1, (playerFuel >> 8), 1);
    //playerFuel -= 20;
}


void get_line(unsigned char* line, unsigned int blocksDownIn) {
    #include "get-line.c"
}

void redraw_all() {
    unsigned char line[28/2];
    unsigned char blocksDown = 2;
    
    for (blocksDown = 2; blocksDown < 28+2; blocksDown+=2) {
        get_line(line, topOfScreenBlocksDown+(blocksDown/2)-1);
        
        unsigned char across = 0;
        for (across = 0; across < 28; across+=2) {
            SetTile(across, blocksDown, line[across/2]);
            SetTile(across+1, blocksDown, line[across/2]+1);
            SetTile(across, blocksDown+1, line[across/2]+2);
            SetTile(across+1, blocksDown+1, line[across/2]+3);
        }
        //WaitVsync(1);
    }

    //topOfScreenBlocksDown -= (28/2)-4;
    //playerX -= 16;
    //ClearVram();
}
/*
#define MENU_PING_HERE TriggerNote(2,31,16,0xA0);
#define MENU_YES_HERE TriggerNote(2,33,16,0xA0);
#define MENU_NO_HERE TriggerNote(2,32,16,0xA0);
*/

#define MENU_PING_HERE TriggerFx(31,0xA0, 1);
#define MENU_YES_HERE TriggerFx(33,0xA0, 1);
#define MENU_NO_HERE TriggerFx(32,0xA0, 1);


void open_shop() {
    #include "shop.c"
}

void draw_depth() {
    int depth = topOfScreenBlocksDown+(playerY/16)-513+2;
    unsigned char minus = 0;
    //From 15 for x to the leftwards.
    if (depth < 0) {
        depth *= -1;
        minus = 1;
    }
    
    unsigned char x = 15;
    do {
        unsigned char digit = depth % 10;
        SetTile(x, 1, TILE_WITH_SCORE_0 + digit);
        x--;
        depth /= 10;
    } while (depth > 0);
    if (minus == 1) {
        SetTile(x, 1, TILE_WITH_SCORE_0-'0'+'-');
        x--;
    }
    SetTile(x, 1, BLACK_TILE);
}

unsigned char missedVsync = 1;

void beforeVsync() {
    PrintInt(0, 10, missedVsync, 1);
    if (missedVsync == 1) {
        missedVsync = 2;
    } else {
        missedVsync = 1;
    }
}

void generateMap() {
    #include "genmap.c"
}



int main() {
    //SetFontTable(image);
    SetTileTable(image);
    //init music too.
    InitMusicPlayer(patches);
    StartSong(midisong);
    SetMasterVolume(100);

    //Do the menu
    #include "menu.c"
    
    StartSong(gamesong);
    
    //////INIT SPRITES
    //Bank to for sprites
    SetSpritesTileBank(0, image);
    
    sprites[0].x = 0;
    sprites[0].y = 0;
    sprites[0].tileIndex = TILE_PLAYER_TOP_LEFT;
    sprites[0].flags = SPRITE_BANK0;

    sprites[1].x = 0;
    sprites[1].y = 0;
    sprites[1].tileIndex = TILE_PLAYER_TOP_RIGHT;
    sprites[1].flags = SPRITE_BANK0;
    
    sprites[2].x = 0;
    sprites[2].y = 0;
    sprites[2].tileIndex = TILE_PLAYER_BOTTOM_LEFT;
    sprites[2].flags = SPRITE_BANK0;
    
    sprites[3].x = 0;
    sprites[3].y = 0;
    sprites[3].tileIndex = TILE_PLAYER_BOTTOM_RIGHT;
    sprites[3].flags = SPRITE_BANK0;
    
    //////END INIT SPRITES
    
    
    
    
    
    ClearVram();
    
    //Fill the overlay
    Fill(0, 0, 28, 2, BLACK_TILE);
    SetTile(0, 0, TILE_ENERGY_LEFT);
    Print(1, 0, PSTR("ENERGY"));
    SetTile(7, 0, TILE_ENERGY_RIGHT);
    
    SetTile(27, 0, TILE_WITH_SCORE_0);
    SetTile(26, 0, TILE_WITH_SCORE_0);
    SetTile(20, 0, TILE_WITH_SCORE_0-'0'+'$');
    //In case we loaded a score
    draw_score();
    
    Print(12, 0, PSTR("DEPTH"));
    SetTile(16, 1, TILE_WITH_SCORE_0-'0'+'m');
    
    //Fill the gap at start
    
    char animationGoing = 0;
    unsigned char aimX = 0;
    unsigned char aimY = 0;
    
    SetUserPreVsyncCallback(&beforeVsync);
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    playerVSpeed = 10;
    playerIsGoingUp = 0;
    SetSpriteVisibility(0);
    redraw_all();
    SetSpriteVisibility(1);
    
    
    //////MAIN GAME LOOP
    char mainLoop = 1;
    while (mainLoop == 1) {
        //Work out if we're on the ground
        unsigned char blockUnderBottomLeft = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE, PLAYER_BOTTOM_SIDE+1, scrollYOffset);
        unsigned char blockUnderBottomRight = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE, PLAYER_BOTTOM_SIDE+1, scrollYOffset);
        char onGround = blockUnderBottomLeft >= TILE_BLOCK || blockUnderBottomRight >= TILE_BLOCK;
        
        //Work out if there's walls to the sides
        unsigned char blockToLeftOfTop = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE-1, PLAYER_TOP_SIDE, scrollYOffset);
        unsigned char blockToRightOfTop = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_TOP_SIDE, scrollYOffset);
        unsigned char blockToLeftOfBottom = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE-1, PLAYER_BOTTOM_SIDE, scrollYOffset);
        unsigned char blockToRightOfBottom = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_BOTTOM_SIDE, scrollYOffset);
        
        char wallOnLeft = blockToLeftOfTop >= TILE_BLOCK || blockToLeftOfBottom >= TILE_BLOCK;
        char wallOnRight = blockToRightOfTop >= TILE_BLOCK || blockToRightOfBottom >= TILE_BLOCK;        
        
        unsigned int joy = ReadJoypad(0);
        if (joy & BTN_START) {
            //pause. This is BBGGGRRR fade mask thing.
            //MAKE VOLUME 0 AND PLAY DING SOUND>
            //Set bits you want to 1.
            DDRC = 0xFF ^ (1 << 3) ^ 3;
            
            Print(12, 15, PSTR("PAUSE"));
            //Don't let them cover the text
            SetSpriteVisibility(0);
            
            while (ReadJoypad(0) & BTN_START) {WaitVsync(1);}
            //They released.
                
            while (true) {
                joy = ReadJoypad(0);
                if (joy & BTN_START) {
                    break;
                }
                WaitVsync(1);
            }
            
            while (ReadJoypad(0) & BTN_START) {WaitVsync(1);}
            //They released.
            //onwards!
            redraw_all();
            SetSpriteVisibility(1);
            DDRC = 0xFF;
        }
        //////PROCESS BUTTON PRESSES
        if (animationGoing == 0) {
            if (joy & BTN_UP) {
                ////////////writeSave();
                //FLY!?
                
                //Nope/ playerFuel -= flyOrMineDrainEnergy;
                if (!playerIsGoingUp) {
                    if (playerVSpeed >= jetpackSpeed) {
                        playerVSpeed -= jetpackSpeed;
                    } else {
                        playerIsGoingUp = 1;
                        //Shouldn't actually be 2 but nvm
                        playerVSpeed = jetpackSpeed-playerVSpeed;
                    }
                } else {
                    //Max flying speed!
                    if (playerVSpeed < maxFlySpeed) {
                        playerVSpeed += jetpackSpeed;
                    } else {
                        playerVSpeed = maxFlySpeed;
                    }
                }
            }
            
            if (joy & BTN_DOWN && onGround) {
                unsigned tempX = ((playerX+8)/16)*16;
                unsigned tempY = playerY + 16;
                //Check if that place is already mined
                if (GET_TILE_AT_X_Y(tempX+8, tempY+8, scrollYOffset) == 0) {
                    //Already mined so mine the other one
                    if (tempX == ((PLAYER_LEFT_SIDE)/16)*16) {
                        tempX += 16;
                    } else {
                        tempX -= 16;
                    }
                }
                
                if (GET_TILE_AT_X_Y(tempX, tempY+5, scrollYOffset) == TILE_ROCK) {
                    //rock on (or off)
                } else {
                    //mine down.
                    animationGoing = 1;
                    aimY = tempY;
                    aimX = tempX;
                }
            }
            if (animationGoing == 0) {
                //not mining so move left or right
                if (joy & BTN_LEFT) {
                    
                    //If there's a wall we mine it
                    if (onGround && wallOnLeft && !(joy & BTN_UP) && playerY == (playerY/8)*8) {
                        unsigned tempX = ((playerX-2)/16)*16;
                        if (GET_TILE_AT_X_Y(tempX, PLAYER_TOP_SIDE+2, scrollYOffset) == TILE_ROCK) {
                            //it's a rock. Do not mine it
                        } else {
                            //mine
                            animationGoing = 1;
                            aimY = playerY;
                            aimX = tempX;
                        }
                    } else {
                        if (!wallOnLeft) {
                            //Move
                            if (playerX >= playerSpeed) {
                                playerX -= playerSpeed;
                            } else {
                                playerX = 0;
                            }
                        } else {
                            if (GET_TILE_AT_X_Y(playerX-1, PLAYER_TOP_SIDE-1, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(playerX-1, PLAYER_BOTTOM_SIDE-1, scrollYOffset) < TILE_BLOCK) {
                                playerX -= 2;
                                playerY -= 1;
                            } else if (GET_TILE_AT_X_Y(playerX-1, PLAYER_TOP_SIDE+2, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(playerX-1, PLAYER_BOTTOM_SIDE+2, scrollYOffset) < TILE_BLOCK) {
                                playerX -= 2;
                                playerY += 2;
                            } else if (GET_TILE_AT_X_Y(playerX-1, PLAYER_TOP_SIDE+1, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(playerX-1, PLAYER_BOTTOM_SIDE+1, scrollYOffset) < TILE_BLOCK) {
                                playerX -= 2;
                                playerY += 1;
                            } 

                        }
                    }
                    //Flip sprite
                    sprites[0].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
                    sprites[1].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
                    sprites[2].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
                    sprites[3].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
                }
                if (joy & BTN_RIGHT) {
                    //If there's a wall we mine it
                    if (onGround && wallOnRight && !(joy & BTN_UP) && playerY == (playerY/8)*8) {
                        unsigned tempX = ((playerX+2)/16)*16+16;
                        if (GET_TILE_AT_X_Y(tempX, PLAYER_TOP_SIDE+2, scrollYOffset) == TILE_ROCK) {
                            //it's a rock. Do not mine it
                        } else {
                            //mine
                            animationGoing = 1;
                            aimY = playerY;
                            aimX = tempX;
                        }
                    } else {
                        if (!wallOnRight) {
                            //Move
                            playerX += playerSpeed;
                            if (playerX > ((28*8)-16)-playerSpeed+2) {
                                playerX = (28*8)-16;
                            }
                        } else {
                            //Wall there so check if were are near a gap
                            if (GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_TOP_SIDE-1, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_BOTTOM_SIDE-1, scrollYOffset) < TILE_BLOCK) {
                                playerX += 2;
                                playerY -= 1;
                            } else if (GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_TOP_SIDE+2, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_BOTTOM_SIDE+2, scrollYOffset) < TILE_BLOCK) {
                                playerX += 2;
                                playerY += 2;
                            } else if (GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_TOP_SIDE+1, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_BOTTOM_SIDE+1, scrollYOffset) < TILE_BLOCK) {
                                playerX += 2;
                                playerY += 1;
                            } 
                        }
                    }
                    //Normal flags
                    sprites[0].flags = SPRITE_BANK0;
                    sprites[1].flags = SPRITE_BANK0;
                    sprites[2].flags = SPRITE_BANK0;
                    sprites[3].flags = SPRITE_BANK0;
                }
                
            }
        } else {
            //We are in an animation
            if (playerFuel <= flyOrMineDrainEnergy) {
                playerFuel = 0;
            } else {
                playerFuel -= flyOrMineDrainEnergy;
            }
            if (animationSpeedHalf != 0 && frameCount & 1) {
                //we add one to speed this frame
                animationSpeed++;
            }
            
            unsigned char goSlow = 0;
            //If over half way down go slower
            if (topOfScreenBlocksDown > 500+(DEPTH/2) && digSpeedLevel != 0) {
                goSlow = 1;
                animationSpeed--;
            }
            
            if (playerX < aimX) {
                playerX += animationSpeed;
                if (playerX > aimX) {
                    playerX = aimX;
                }
            } else if (playerX > aimX) {
                playerX -= animationSpeed;
                if (playerX < aimX) {
                    playerX = aimX;
                }
            }
            
            if (playerY < aimY) {
                playerY += animationSpeed;
                if (playerY > aimY) {
                    playerY = aimY;
                }
            } else if (playerY > aimY) {
                playerY -= animationSpeed;
                if (playerY < aimY) {
                    playerY = aimY;
                }
            }
            
            if (playerX == aimX && playerY == aimY) {
                //Add to score
                unsigned char tileType = GET_TILE_AT_X_Y(aimX+8, aimY+8, scrollYOffset);
                if (tileType >= TILE_BLOCK && tileType != TILE_ROCK) {
                    tileType = (tileType - TILE_BLOCK)/4;
                    playerHoldingScore += lookupForMoneyForTileByOrderInImage[tileType];
                    draw_hold_score();

                    //Finally clear the spaces and store them as mined.
                    SetTile((playerX/8), ((playerY+scrollYOffset)/8), 0);
                    SetTile((playerX/8), ((playerY+scrollYOffset)/8)+1, 0);
                    SetTile((playerX/8)+1, ((playerY+scrollYOffset)/8), 0);
                    SetTile((playerX/8)+1, ((playerY+scrollYOffset)/8)+1, 0);
                    unsigned int bitWeNeed = (playerX/16)+((topOfScreenBlocksDown-500)+(playerY)/16)*(28/2)-(24*8)-4+((28/2)*2);
                    
                    tunneledArray[bitWeNeed/8] |= 1 << bitWeNeed%8;
                    //PrintInt(9, 6, bitWeNeed/8 ,1);
                }
                animationGoing = 0;
                playerVSpeed = 0;
            }
            
            if (goSlow == 1) {
                animationSpeed++; //restore
            }
            
            if (animationSpeedHalf != 0 && frameCount & 1) {
                //I'm sure there's a nicer way to do this.
                animationSpeed--;
            }
        }
        //////END PROCESS BUTTON PRESSES
        //Animate!
        if (joy & (BTN_RIGHT | BTN_LEFT) || animationGoing) {
            //if left or right is pressed do animation
            //set sprite
            if (onGround && frameCount & (1 << 3)) {
                if (aimY > playerY && animationGoing) {
                    sprites[0].tileIndex = TILE_PLAYER_DIG_TOP_LEFT;
                    sprites[1].tileIndex = TILE_PLAYER_DIG_TOP_RIGHT;
                    sprites[2].tileIndex = TILE_PLAYER_DIG_BOTTOM_LEFT;
                    sprites[3].tileIndex = TILE_PLAYER_DIG_BOTTOM_RIGHT;
                } else {
                    sprites[0].tileIndex = TILE_PLAYER_WALK_TOP_LEFT;
                    sprites[1].tileIndex = TILE_PLAYER_WALK_TOP_RIGHT;
                    sprites[2].tileIndex = TILE_PLAYER_WALK_BOTTOM_LEFT;
                    sprites[3].tileIndex = TILE_PLAYER_WALK_BOTTOM_RIGHT;
                }
            } else {
                sprites[0].tileIndex = TILE_PLAYER_TOP_LEFT;
                sprites[1].tileIndex = TILE_PLAYER_TOP_RIGHT;
                sprites[2].tileIndex = TILE_PLAYER_BOTTOM_LEFT;
                sprites[3].tileIndex = TILE_PLAYER_BOTTOM_RIGHT;
            }
       } else {
            sprites[0].tileIndex = TILE_PLAYER_TOP_LEFT;
            sprites[1].tileIndex = TILE_PLAYER_TOP_RIGHT;
            sprites[2].tileIndex = TILE_PLAYER_BOTTOM_LEFT;
            sprites[3].tileIndex = TILE_PLAYER_BOTTOM_RIGHT;
        }
        
        if (joy & BTN_UP && (frameCount & (1 << 2))) {
            sprites[0].tileIndex = TILE_PLAYER_TOP_LEFT_FIRE;
            sprites[1].tileIndex = TILE_PLAYER_TOP_RIGHT_FIRE;
            sprites[2].tileIndex = TILE_PLAYER_BOTTOM_LEFT_FIRE;
            sprites[3].tileIndex = TILE_PLAYER_BOTTOM_RIGHT_FIRE;
            TriggerNote(3,30,10,20);
        }
        
        
        
        //////APPLY GRAVITY
        if (animationGoing == 0) {
            unsigned char speedAsInt = (playerVSpeed >> 3);
            if (playerIsGoingUp == 1) {
                //We are moving up
                //collisions above are possible playerVSpeed playerIsGoingUp
                unsigned char blockOverTopLeft = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE, PLAYER_TOP_SIDE-speedAsInt, scrollYOffset);
                unsigned char blockOverTopRight = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE, PLAYER_TOP_SIDE-speedAsInt, scrollYOffset);
                char justUnderBlock = blockOverTopLeft >= TILE_BLOCK || blockOverTopRight >= TILE_BLOCK;
                
                if (justUnderBlock) {
                    //Check if nearly correct
                    if (GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE-2, PLAYER_TOP_SIDE-speedAsInt, scrollYOffset) < TILE_BLOCK && blockOverTopLeft < TILE_BLOCK) {
                        playerX -= 2;
                        playerY -= speedAsInt;
                        playerVSpeed -= 1;
                    } else if (GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE+2, PLAYER_TOP_SIDE-speedAsInt, scrollYOffset) < TILE_BLOCK && blockOverTopRight < TILE_BLOCK) {
                        playerX += 2;
                        playerY -= speedAsInt;
                        playerVSpeed -= 1;
                    } else {
                        playerVSpeed = 0;
                        playerY = ((playerY+scrollYOffset)/16)*16-scrollYOffset;
                    }
                } else {
                    playerY -= speedAsInt;
                    playerVSpeed -= 1;
                }
                if (playerVSpeed == 0) {
                    playerIsGoingUp = 0;
                }
            } else {
                //Going down so check if we've hit the floor
                playerY += speedAsInt;
                if (GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE, PLAYER_BOTTOM_SIDE+1, scrollYOffset) >= TILE_BLOCK || GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE, PLAYER_BOTTOM_SIDE+1, scrollYOffset) >= TILE_BLOCK) {
                    if (GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE+1, PLAYER_BOTTOM_SIDE+1, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_BOTTOM_SIDE+1, scrollYOffset) < TILE_BLOCK) {
                        playerX += 1;
                        if (playerVSpeed < (5 << 3)-1) {
                            playerVSpeed += 1;
                        }
                        playerY += 2;
                    } else if (GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE-1, PLAYER_BOTTOM_SIDE+1, scrollYOffset) < TILE_BLOCK && GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE-1, PLAYER_BOTTOM_SIDE+1, scrollYOffset) < TILE_BLOCK) {
                        playerX -= 1;
                        if (playerVSpeed < (5 << 3)-1) {
                            playerVSpeed += 1;
                        }
                        playerY += 2;
                    } else {
                        playerVSpeed = 0;
                        playerY = ((playerY+scrollYOffset)/16)*16-scrollYOffset;
                    }
                } else {
                    //playerY += speedAsInt;
                    //apply gravity and don't go over the max fall speed
                    if (playerVSpeed < (5 << 3)-1) {
                        playerVSpeed += 1;
                    }
                }
            }
        }
        //////END APPLY GRAVITY
        
        
        //Draw player very early in case we don't finish this frame properly
        //draw_player();
        #include "scroll.c"
        
        
        unsigned char yourBlock = GET_TILE_AT_X_Y(playerX+8, PLAYER_TOP_SIDE, scrollYOffset);
        if (yourBlock == MAGIC_TILE_FOR_FUEL) {
            playerScore += playerHoldingScore;
            if (playerHoldingScore != 0) {
                MENU_YES_HERE
            }
            playerHoldingScore = 0;
            draw_score();
            playerFuel = (255 << 8) + 255;
        } else if (yourBlock == MAGIC_TILE_FOR_SHOP) {
            open_shop();
        }
        
        //if (yourBlock == MAGIC_TILE_FOR_SAVE) {
        if (topOfScreenBlocksDown < 500-10) {
            SetSpriteVisibility(0);
            topOfScreenBlocksDown = 500-6;
            Fill(5, 12, 18, 6, BLACK_TILE);
            Print(4+3, 13, PSTR("Would%you%like"));
            Print(4+3+3, 14, PSTR("to%save?"));
            Print(9+1, 16, PSTR("Yes%%%No"));
            unsigned char selectedYes = 0;
            unsigned char waitingForRelease = 0;
            while (1 == 1) {
                if (selectedYes == 1) {
                    SetTile(8+1, 16, TILE_WITH_MENU_ARROW);
                    SetTile(8+1+6, 16, BLACK_TILE);
                } else {
                    SetTile(8+1+6, 16, TILE_WITH_MENU_ARROW);
                    SetTile(8+1, 16, BLACK_TILE);
                }
                int joy = ReadJoypad(0);
                if (joy & BTN_LEFT && waitingForRelease == 0) {
                    selectedYes ^= 1;
                    waitingForRelease = 1;
                    MENU_PING_HERE
                }
                if (joy & BTN_RIGHT && waitingForRelease == 0) {
                    selectedYes ^= 1;
                    waitingForRelease = 1;
                    MENU_PING_HERE
                }
                if (joy == 0) {
                    waitingForRelease = 0;
                }
                if (joy & BTN_A) {
                    if (selectedYes == 1) {
                        MENU_YES_HERE
                        //SAVING! WHEN HIGH UP
                        Fill(5, 12, 18, 6, BLACK_TILE);
                        Print(4+4, 14, PSTR("Saving..."));
                        WaitVsync(60);
                        writeSave();
                        break;
                    } else {
                        //exit
                        MENU_NO_HERE
                        break;
                    }
                }
                if (joy & BTN_B) {
                    //exit
                    MENU_NO_HERE
                    break;
                }
            }
            redraw_all();
            playerVSpeed = 20;
            playerIsGoingUp = 0;
            SetSpriteVisibility(1);
        }
        
        if (playerFuel <= idleDrainEnergy) {
            playerFuel = 0;
        } else {
            playerFuel -= idleDrainEnergy;
        }
        draw_fuel();
        draw_player();
        draw_depth();

        //PrintInt(10, 0, topOfScreenBlocksDown, 1);
        
        //MAKE SURE THAT YOU CAN@T LOSE MORE THAN THIS AMOUNT OF FUEL IN ONE GO AND NOT DIE! (via underflow)
        if (playerFuel == 0) {
            //playerFuel = 0;
            //draw_fuel();
            //game over
            SetSpriteVisibility(0);
            Fill((28/2)-6, (28/2)-1, 12, 3, 5);
            Print((28/2)-5, (28/2), PSTR("Game%over."));
            unsigned char vol = 100;
            while (vol > 0) {
                SetMasterVolume(vol);
                vol--;
                WaitVsync(2);
            }
            
            WaitVsync(60*10);
            SetSpriteVisibility(1);
            SetMasterVolume(100);
            //The SoftReset() function doesn't seem to work on the emulator.
            //I don't think the jump to 0 is an amazing choice but it seems to work for now.
            void (*toTheStart)(void);
            toTheStart = (void (*)(void))0;
            toTheStart();
            SoftReset();
            return 0;
        }
        //PrintBinaryByte(5, 5, tunneledArray[0]);
        frameCount++;

        /*if (missedVsync == 2) {
            WaitVsync(10);
        }*/
        missedVsync = 0;
        WaitVsync(1);
    }
    //////END MAIN GAME LOOP

    return 0;
}
