#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <uzebox.h>

#include "data/image.inc"

#define SCREEN_WIDTH 240
#define SCREEN_HEIGHT 224
#define TILES_ON_X (SCREEN_WIDTH/8)
#define TILES_ON_Y (SCREEN_HEIGHT/8)
//30 and 28.
//A pixel is a byte. 8*8 = 64 bytes in an image
#define BYTES_IN_TILE 64


//player image sprites
#define TILE_PLAYER_TOP_LEFT 96
#define TILE_PLAYER_TOP_RIGHT 97
#define TILE_PLAYER_BOTTOM_LEFT 100
#define TILE_PLAYER_BOTTOM_RIGHT 101

#define TILE_BLOCK 98

//some fake vars for the player
#define playerX (sprites[0].x)
#define playerY (sprites[0].y)

#define scrollYOffset (Screen.scrollY)



#define PLAYER_MOVE_LEFT_SPEED 2
#define PLAYER_MOVE_RIGHT_SPEED 2
#define PLAYER_MOVE_UP_SPEED 2
#define PLAYER_MOVE_DOWN_SPEED 2

//I'm pretty sure I don't need to make a global variable be set to 0?
//the var below is a bool by the way
char playerIsGoingUp = 0;
unsigned char playerVSpeed = 0x00;

void draw_player() {
    //we already have acess to playerX and playerY
    //sprites[0].x = playerX;
    //sprites[0].y = playerY;
    //top right
    sprites[1].x = playerX+8;
    sprites[1].y = playerY;
    //bottom left
    sprites[2].x = playerX;
    sprites[2].y = playerY+8;
    //bottom right
    sprites[3].x = playerX+8;
    sprites[3].y = playerY+8;
}


int main() {
    //SetFontTable(image);
    SetTileTable(image);
    //init music too.

    ClearVram();

    ////////START MENU
    while (true) {
        Print(4 ,13, PSTR("Press any key to start"));
        //Could include this line below but pointless for now I guess
        //WaitVsync();
        //get joy 1
        int joy = ReadJoypad(0);
        if (joy != 0) {
            break;
        }
    }
    //////END MENU

    
    //////INIT SPRITES
    //Bank to for sprites
    SetSpritesTileBank(0, image);
    
    sprites[0].x = 0;
    sprites[0].y = 0;
    sprites[0].tileIndex = TILE_PLAYER_TOP_LEFT;
    sprites[0].flags = SPRITE_BANK0;

    sprites[1].x = 0;
    sprites[1].y = 0;
    sprites[1].tileIndex = TILE_PLAYER_TOP_RIGHT;
    sprites[1].flags = SPRITE_BANK0;
    
    sprites[2].x = 0;
    sprites[2].y = 0;
    sprites[2].tileIndex = TILE_PLAYER_BOTTOM_LEFT;
    sprites[2].flags = SPRITE_BANK0;
    
    sprites[3].x = 0;
    sprites[3].y = 0;
    sprites[3].tileIndex = TILE_PLAYER_BOTTOM_RIGHT;
    sprites[3].flags = SPRITE_BANK0;
    //////END INIT SPRITES
    
    ClearVram();
    Fill(0, 20, 28, 10, TILE_BLOCK);
    
    char playerIsFalling = 1;
    int dsklhfdslkafjdsahfjksadghfkjsdghflasdhflksdh = 0;
    //////MAIN GAME LOOP
    char mainLoop = 1;
    while (mainLoop == 1) {
        //ClearVram();
        
        //////PROCESS BUTTON PRESSES
        {
            int joy = ReadJoypad(0);
            if (joy & BTN_LEFT) {
                playerX -= PLAYER_MOVE_LEFT_SPEED;
            }
            if (joy & BTN_RIGHT) {
                playerX += PLAYER_MOVE_RIGHT_SPEED;
            }
            if (joy & BTN_UP) {
                //playerY -= PLAYER_MOVE_UP_SPEED;
                unsigned char blockAtBottomLeft = GetTile((playerX/8), (playerY/8)+2);
                unsigned char blockAtBottomRight = GetTile((playerX/8)+1, (playerY/8)+2);
                if (blockAtBottomLeft == TILE_BLOCK || blockAtBottomRight == TILE_BLOCK) {
                    playerIsGoingUp = 1;
                    playerVSpeed = 35;
                }
            }
            if (joy & BTN_DOWN) {
                SetTile(playerX/8, ((playerY+16)/8), 0);
                
                SetTile(playerX/8+1, ((playerY+16)/8), 0);
                SetTile(playerX/8, ((playerY+16)/8)+1, 0);
                SetTile(playerX/8+1, ((playerY+16)/8)+1, 0);
            }
        }
        //////END PROCESS BUTTON PRESSES
        
        
        //////Correct possition for walls
        {
            unsigned char blockAtTopLeft = GetTile((playerX/8), (playerY/8));
            unsigned char blockAtTopRight = GetTile(((playerX+15)/8), (playerY/8));
            unsigned char blockAtBottomLeft = GetTile((playerX/8), ((playerY+15)/8));
            unsigned char blockAtBottomRight = GetTile(((playerX+15)/8), ((playerY+15)/8));
/*
            if (blockAtTopLeft == TILE_BLOCK || blockAtBottomLeft == TILE_BLOCK) {
                playerX += PLAYER_MOVE_LEFT_SPEED;
            }
            if (blockAtTopRight == TILE_BLOCK || blockAtBottomRight == TILE_BLOCK) {
                playerX -= PLAYER_MOVE_RIGHT_SPEED;
            }*/
        }
        //////end correction
        
        
        //////APPLY GRAVITY
        {
            unsigned char speedAsInt = (playerVSpeed >> 3);
            
            if (playerIsGoingUp == 1) {
                playerY -= speedAsInt;
                //apply gravity
                playerVSpeed -= 1;
                if (playerVSpeed == 0) {
                    playerIsGoingUp = 0;
                }
            } else {
                playerY += speedAsInt;
                //apply gravity and don't go over the max fall speed
                if (playerVSpeed < 0x15) {
                    playerVSpeed += 1;
                }
            }
        }
        //////END APPLY GRAVITY
        
        
        //////Correct for collision
        {
            unsigned char blockAtTopLeft = GetTile((playerX/8), (playerY/8));
            unsigned char blockAtTopRight = GetTile(((playerX+15)/8), (playerY/8));
            unsigned char blockAtBottomLeft = GetTile((playerX/8), (((playerY-scrollYOffset)+15)/8));
            unsigned char blockAtBottomRight = GetTile(((playerX+15)/8), (((playerY-scrollYOffset)+15)/8));
            playerIsFalling = 1;
            /*if (blockAtTopLeft == TILE_BLOCK || blockAtTopRight == TILE_BLOCK) {
                playerY = (playerY/8)*8-8;
                playerVSpeed = 0;
                playerIsFalling = 0;
            }*/
            if (blockAtBottomLeft == TILE_BLOCK || blockAtBottomRight == TILE_BLOCK) {
                playerY = (playerY/8)*8;
                playerVSpeed = 0;
                playerIsFalling = 0;
            }
        }
        //////End correction

        
        //Draw player very early in case we don't finish this frame properly
        draw_player();
        
        //////Scroll screen
        if (playerY > (8*22)) {

            //We need to scroll down
            unsigned char amountExtraToScroll = playerY - (8*22);
            
            
            
            
            
            
            //Try to hide lag by redrawing very early
            playerY -= amountExtraToScroll;
            draw_player();
            
            scrollYOffset += amountExtraToScroll;
            if (scrollYOffset >= 16) {
                scrollYOffset -= 16;
                
                //memcpy(&vram[0], &vram[256], 256);
                //memcpy(&vram[256], &vram[256+256], 256);
                //memcpy(&vram[256+256], &vram[256+256+256], 256);
                
                
                /*int lineCount, rowCount;
                for (lineCount = 0; lineCount < 28; lineCount++) {
                    for (rowCount = 0; rowCount < 28; rowCount++) {
                        SetTile(rowCount, lineCount, GetTile(rowCount, lineCount+2));
                    }
                }*/
                //To make x scrolling possible (which I don't even use)
                //all of the vram is kind of placed randomly.
                //We move them all up a double tile
                //0 is top left and then it goes down 8 tiles and then along one.
                //Top chunk
                //This line would move left one block
                //memcpy(vram[0], vram[8], foo);
                int tmp = 0;
                for (tmp=0; tmp<28; tmp++) {
                    //This moves one bit up
                    //memcpy(vram[0], vram[2], 6);
                    
                    //top chunk, remove the top two lines
                    memmove(&vram[0+tmp*8], &vram[tmp*8+2], 6);
                    
                    //Next chunk, move into top
                    memmove(&vram[0+tmp*8+6], &vram[tmp*8+256], 2);
                    memmove(&vram[0+tmp*8+256], &vram[tmp*8+256+2], 6);
                    
                    memmove(&vram[0+tmp*8+256+6], &vram[tmp*8+256+256], 2);
                    memmove(&vram[0+tmp*8+256+256], &vram[tmp*8+256+256+2], 6);
                    
                    memmove(&vram[0+tmp*8+256+256+6], &vram[tmp*8+256+256+256], 2);
                    memmove(&vram[0+tmp*8+256+256+256], &vram[tmp*8+256+256+256+2], 6);
                }
            }
           /* if (scrollYOffset >= 16) {
                scrollYOffset -= 16;
                //We need to draw a new line
                unsigned char rowCount = 0;
                unsigned char lineCount = 0;
                for (; lineCount < 6; lineCount++) {
                    for (rowCount = 0; rowCount < 28; rowCount++) {
                        SetTile(rowCount, lineCount, GetTile(rowCount, lineCount+1));
                    }
                }*/
                //Fill(0, 28, 28, 2, 0);
            /*}*/
            ///////////////////////playerY -= amountExtraToScroll;
            //playerY = 8*22;
        }
        //////end scroll screen
        
        
        //dsklhfdslkafjdsahfjksadghfkjsdghflasdhflksdh = 256+256+256+3;
        //vram[dsklhfdslkafjdsahfjksadghfkjsdghflasdhflksdh] = TILE_BLOCK;
        //dsklhfdslkafjdsahfjksadghfkjsdghflasdhflksdh++;
        //int tmp = 0;
        //for (tmp=0; tmp<28; tmp++) {
        //    memmove(&vram[256+256+256], &vram[256+256+256+2], 6);
        //}
            
        //memcpy(&vram[256+256], &vram[256+256+256], 256);
        
        //Draw everything
        ////////////////////////////draw_player();
        WaitVsync(1);
    }
    //////END MAIN GAME LOOP

    return 0;
}
