#include <avr/io.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <uzebox.h>
//for memmove
#include <string.h>

#include "data/image.inc"

#define SCREEN_WIDTH 240
#define SCREEN_HEIGHT 224
#define TILES_ON_X (SCREEN_WIDTH/8)
#define TILES_ON_Y (SCREEN_HEIGHT/8)
//30 and 28.
//A pixel is a byte. 8*8 = 64 bytes in an image
#define BYTES_IN_TILE 64


//player image sprites
#define TILE_PLAYER_TOP_LEFT 96
#define TILE_PLAYER_TOP_RIGHT 97
#define TILE_PLAYER_BOTTOM_LEFT 100
#define TILE_PLAYER_BOTTOM_RIGHT 101

#define TILE_BLOCK 98
#define TILE_BLOCK_2 103
#define TILE_BLOCK_ORE_1 104

unsigned char playerX = 50;
unsigned char playerY = 50;

#define scrollYOffset (Screen.scrollY)


#define PLAYER_LEFT_SIDE (playerX+2)
#define PLAYER_RIGHT_SIDE (playerX+15-2)
#define PLAYER_BOTTOM_SIDE (playerY+15)
#define PLAYER_TOP_SIDE (playerY)

#define PLAYER_TOP (playerY)

#define GET_TILE_AT_X_Y(x,y,scrollYOffset) (GetTile(((x)/8), (((y)+(scrollYOffset))/8)))



#define PLAYER_MOVE_LEFT_SPEED 2
#define PLAYER_MOVE_RIGHT_SPEED 2
#define PLAYER_MOVE_UP_SPEED 2
#define PLAYER_MOVE_DOWN_SPEED 2
#define PLAYER_ANIMATION_STEP 2

//I'm pretty sure I don't need to make a global variable be set to 0?
//the var below is a bool by the way
char playerIsGoingUp = 0;
unsigned char playerVSpeed = 0x10;
int topOfScreenBlocksDown = 0;

void draw_player() {
    //If it's not fliped
    if (sprites[0].flags == SPRITE_BANK0) {
        //They are facing right
        //Top left
        sprites[0].x = playerX;
        sprites[0].y = playerY;
        //top right
        sprites[1].x = playerX+8;
        sprites[1].y = playerY;
        //bottom left
        sprites[2].x = playerX;
        sprites[2].y = playerY+8;
        //bottom right
        sprites[3].x = playerX+8;
        sprites[3].y = playerY+8;
    } else {
        //We need to swap our sprites around to look correct (they're also flipped on x)
        sprites[0].x = playerX+8;
        sprites[0].y = playerY;
        
        sprites[1].x = playerX;
        sprites[1].y = playerY;
        
        //top left with original top right sprite
        sprites[1].x = playerX;
        sprites[1].y = playerY;
        //bottom right with bottom left sprite
        sprites[2].x = playerX+8;
        sprites[2].y = playerY+8;
        //bottom left with bottom right sprite
        sprites[3].x = playerX;
        sprites[3].y = playerY+8;
    }
}


int main() {
    //SetFontTable(image);
    SetTileTable(image);
    //init music too.

    ClearVram();

    ////////START MENU
    while (true) {
        Print(4 ,13, PSTR("Press any key to start"));
        //Could include this line below but pointless for now I guess
        //WaitVsync();
        //get joy 1
        int joy = ReadJoypad(0);
        if (joy != 0) {
            break;
        }
    }
    //////END MENU

    
    //////INIT SPRITES
    //Bank to for sprites
    SetSpritesTileBank(0, image);
    
    sprites[0].x = 0;
    sprites[0].y = 0;
    sprites[0].tileIndex = TILE_PLAYER_TOP_LEFT;
    sprites[0].flags = SPRITE_BANK0;

    sprites[1].x = 0;
    sprites[1].y = 0;
    sprites[1].tileIndex = TILE_PLAYER_TOP_RIGHT;
    sprites[1].flags = SPRITE_BANK0;
    
    sprites[2].x = 0;
    sprites[2].y = 0;
    sprites[2].tileIndex = TILE_PLAYER_BOTTOM_LEFT;
    sprites[2].flags = SPRITE_BANK0;
    
    sprites[3].x = 0;
    sprites[3].y = 0;
    sprites[3].tileIndex = TILE_PLAYER_BOTTOM_RIGHT;
    sprites[3].flags = SPRITE_BANK0;
    //////END INIT SPRITES
    
    ClearVram();
    Fill(0, 20, 28, 10, TILE_BLOCK);
    
    
    
    char animationGoing = 0;
    unsigned char aimX = 0;
    unsigned char aimY = 0;
    
    //////MAIN GAME LOOP
    char mainLoop = 1;
    while (mainLoop == 1) {
        
        /*
        //Work out if we're on the ground
        unsigned char blockUnderBottomLeft = GetTile((playerX/8), ((playerY+scrollYOffset)/8)+2);
        unsigned char blockUnderBottomRight = GetTile((playerX/8)+1, ((playerY+scrollYOffset)/8)+2);
        char onGround = blockUnderBottomLeft >= TILE_BLOCK || blockUnderBottomRight >= TILE_BLOCK;
        
        //collisions above are also possible
        unsigned char blockOverTopLeft = GetTile(((playerX+2)/8), (((playerY-3)+scrollYOffset)/8));
        unsigned char blockOverTopRight = GetTile(((playerX-2)/8)+1, (((playerY-3)+scrollYOffset)/8));
        char justUnderBlock = blockOverTopLeft >= TILE_BLOCK || blockOverTopRight >= TILE_BLOCK;
        
        //Work out if there's walls to the sides
        unsigned char blockToLeftOfTop = GetTile(((playerX)/8), ((playerY+scrollYOffset)/8));
        unsigned char blockToRightOfTop = GetTile(((playerX-2)/8)+2, ((playerY+scrollYOffset)/8));
        unsigned char blockToLeftOfBottom = GetTile(((playerX)/8), ((playerY+scrollYOffset)/8)+1);
        unsigned char blockToRightOfBottom = GetTile(((playerX-2)/8)+2, ((playerY+scrollYOffset)/8)+1);
        
        char wallOnLeft = blockToLeftOfTop >= TILE_BLOCK || blockToLeftOfBottom >= TILE_BLOCK;
        char wallOnRight = blockToRightOfTop >= TILE_BLOCK || blockToRightOfBottom >= TILE_BLOCK;
        */
        
        //Work out if we're on the ground
        unsigned char blockUnderBottomLeft = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE, PLAYER_BOTTOM_SIDE+1, scrollYOffset);
        unsigned char blockUnderBottomRight = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE, PLAYER_BOTTOM_SIDE+1, scrollYOffset);
        char onGround = blockUnderBottomLeft >= TILE_BLOCK || blockUnderBottomRight >= TILE_BLOCK;
        
        //Work out if there's walls to the sides
        unsigned char blockToLeftOfTop = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE-1, PLAYER_TOP_SIDE, scrollYOffset);
        unsigned char blockToRightOfTop = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_TOP_SIDE, scrollYOffset);
        unsigned char blockToLeftOfBottom = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE-1, PLAYER_BOTTOM_SIDE, scrollYOffset);
        unsigned char blockToRightOfBottom = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE+1, PLAYER_BOTTOM_SIDE, scrollYOffset);
        
        char wallOnLeft = blockToLeftOfTop >= TILE_BLOCK || blockToLeftOfBottom >= TILE_BLOCK;
        char wallOnRight = blockToRightOfTop >= TILE_BLOCK || blockToRightOfBottom >= TILE_BLOCK;        
        
        
        //////PROCESS BUTTON PRESSES
        if (animationGoing == 0) {
            int joy = ReadJoypad(0);
            
            //PROCESS REQUESTS TO MINE DOWN OR JUMP
            if (onGround) {
                if (joy & BTN_UP) {
                    //Jump
                    playerIsGoingUp = 1;
                    playerVSpeed = 35;
                    //playerVSpeed = 49;
                }
                if (joy & BTN_DOWN) {
                    //Mine down
                    animationGoing = 1;
                    aimY = (playerY/8)*8+16;
                    aimX = ((playerX+2)/16)*16;
                }
            }
            // END OF REQUESTS TO MINE DOWN OR JUMP
            
            if (joy & BTN_LEFT) {
                //If there's a wall we mine it
                if (onGround && wallOnLeft) {
                    //mine
                    animationGoing = 1;
                    aimY = playerY;
                    aimX = ((playerX)/16)*16;
                } else {
                    if (!wallOnLeft) {
                        //Move
                        playerX -= PLAYER_MOVE_LEFT_SPEED;
                        if (playerX > 250) {
                            playerX = 0; //THIS SHOULD BE -2 IF I COULD DO THAT!
                        }
                    }
                }
                //Flip sprite
                sprites[0].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
                sprites[1].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
                sprites[2].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
                sprites[3].flags = SPRITE_BANK0 | SPRITE_FLIP_X;
            }
            if (joy & BTN_RIGHT) {
                //If there's a wall we mine it
                if (onGround && wallOnRight) {
                    //mine
                    animationGoing = 1;
                    aimY = playerY;
                    aimX = ((playerX+2)/16)*16+16;
                } else {
                    if (!wallOnRight) {
                        //Move
                        playerX += PLAYER_MOVE_RIGHT_SPEED;
                        if (playerX > ((28*8)-16)-PLAYER_MOVE_RIGHT_SPEED+2) {
                            playerX = (28*8)-16+2;
                        }
                    }
                }
                //Normal flags
                sprites[0].flags = SPRITE_BANK0;
                sprites[1].flags = SPRITE_BANK0;
                sprites[2].flags = SPRITE_BANK0;
                sprites[3].flags = SPRITE_BANK0;
            }
        } else {
            //We are in an animation
            if (playerX < aimX) {
                playerX += PLAYER_ANIMATION_STEP;
                if (playerX > aimX) {
                    playerX = aimX;
                }
            } else if (playerX > aimX) {
                playerX -= PLAYER_ANIMATION_STEP;
                if (playerX < aimX) {
                    playerX = aimX;
                }
            }
            
            if (playerY < aimY) {
                playerY += PLAYER_ANIMATION_STEP;
                if (playerY > aimY) {
                    playerY = aimY;
                }
            } else if (playerY > aimY) {
                playerY -= PLAYER_ANIMATION_STEP;
                if (playerY < aimY) {
                    playerY = aimY;
                }
            }
            
            if (playerX == aimX && playerY == aimY) {
                animationGoing = 0;
                playerVSpeed = 0;
                SetTile((playerX/8), ((playerY+scrollYOffset)/8), 0);
                SetTile((playerX/8), ((playerY+scrollYOffset)/8)+1, 0);
                SetTile((playerX/8)+1, ((playerY+scrollYOffset)/8), 0);
                SetTile((playerX/8)+1, ((playerY+scrollYOffset)/8)+1, 0);
            }
        }
        //////END PROCESS BUTTON PRESSES
        
        
        //////APPLY GRAVITY
        if (animationGoing == 0) {
            unsigned char speedAsInt = (playerVSpeed >> 3);
            if (playerIsGoingUp == 1) {
                //We are moving up
                //collisions above are possible playerVSpeed playerIsGoingUp
                unsigned char blockOverTopLeft = GET_TILE_AT_X_Y(PLAYER_LEFT_SIDE, PLAYER_TOP_SIDE-speedAsInt, scrollYOffset);
                unsigned char blockOverTopRight = GET_TILE_AT_X_Y(PLAYER_RIGHT_SIDE, PLAYER_TOP_SIDE-speedAsInt, scrollYOffset);
                char justUnderBlock = blockOverTopLeft >= TILE_BLOCK || blockOverTopRight >= TILE_BLOCK;
                
                if (justUnderBlock) {
                    playerVSpeed = 0;
                    playerY = (playerY/16)*16+scrollYOffset;
                } else {
                    playerY -= speedAsInt;
                    playerVSpeed -= 1;
                }
                if (playerVSpeed == 0) {
                    playerIsGoingUp = 0;
                }
            } else {
                //Going down so check if we've hit the floor
                if (!onGround) {
                    playerY += speedAsInt;
                    //apply gravity and don't go over the max fall speed
                    if (playerVSpeed < 0x15) {
                        playerVSpeed += 1;
                    }
                } else {
                    //If your on the ground you don't move.
                    playerVSpeed = 0;
                }
            }
        }
        //////END APPLY GRAVITY
        
        
        //Draw player very early in case we don't finish this frame properly
        draw_player();

        
        //////SCROLL SCREEN
        //Scroll down
        if (playerY > 8*(28-9)) {
            unsigned char amountExtraToScroll = playerY - (8*(28-9));

            //Try to hide lag by redrawing very early
            playerY = 8*(28-9);
            draw_player();
            
            aimY -= amountExtraToScroll;

            scrollYOffset += amountExtraToScroll;
            if (scrollYOffset >= 16) {
                scrollYOffset -= 16;
                topOfScreenBlocksDown++;
                
                //The vram starts with 0 at the top left and then goes down 8 and then along one.
                //In this way we have horizontal chunks of 8 tiles.
                //Obviously this makes them a pain to cycle upwards.
                char columnCount = 0;
                for (columnCount = 0; columnCount < 28; columnCount++) {
                    //top chunk, remove the top two lines
                    memmove(&vram[columnCount*8], &vram[columnCount*8+2], 6);
                    //Next chunk, move into top. We must wrap upwards between chunks too.
                    memmove(&vram[columnCount*8+6], &vram[columnCount*8+256], 2);
                    memmove(&vram[columnCount*8+256], &vram[columnCount*8+256+2], 6);
                    memmove(&vram[columnCount*8+256+6], &vram[columnCount*8+256+256], 2);
                    memmove(&vram[columnCount*8+256+256], &vram[columnCount*8+256+256+2], 6); 
                    memmove(&vram[columnCount*8+256+256+6], &vram[columnCount*8+256+256+256], 2);
                    memmove(&vram[columnCount*8+256+256+256], &vram[columnCount*8+256+256+256+2], 6);
                }
                //After doing this there's a gap of 2 blocks just under the screen to fill.
                unsigned int seed = topOfScreenBlocksDown;
                char iii = 0;
                for (iii= 0; iii < 28; iii+=2) {
                    seed ^= (seed << 13);
                    seed ^= (seed >> 9);
                    seed ^= (seed << 7);
                    unsigned char blockID = (seed >> 12) + TILE_BLOCK;
                    
                    SetTile(iii, 28, blockID);
                    SetTile(iii, 29, blockID);
                    SetTile(iii+1, 28, blockID);
                    SetTile(iii+1, 29, blockID);
                }
            }
        } else if (playerY < 8*3) {
            unsigned char amountExtraToScroll = 8*3 - playerY;
            signed char negAmountToScroll = scrollYOffset - amountExtraToScroll;
            //Try to hide lag by redrawing very early
            playerY = 8*3;
            draw_player();
            
            aimY += amountExtraToScroll;

            if (negAmountToScroll < 0) {
                scrollYOffset = negAmountToScroll + 16;
                topOfScreenBlocksDown--;
                
                //The vram starts with 0 at the top left and then goes down 8 and then along one.
                //In this way we have horizontal chunks of 8 tiles.
                //Obviously this makes them a pain to cycle upwards.
                char columnCount = 0;
                
                for (columnCount = 0; columnCount < 28; columnCount++) {
                    //bottom chunk, remove the bottom two lines
                    //NOTE TO SELF, I COULD USE MEMCPY INSTEAD OF MEMMOVE FOR SOME OF THESE
                    //Shift down 2
                    memmove(&vram[columnCount*8+256+256+256+2], &vram[columnCount*8+256+256+256], 6);
                    memmove(&vram[columnCount*8+256+256+256], &vram[columnCount*8+256+256+6], 2);
                    
                    memmove(&vram[columnCount*8+256+256+2], &vram[columnCount*8+256+256], 6);
                    memmove(&vram[columnCount*8+256+256], &vram[columnCount*8+256+6], 2);
                    
                    memmove(&vram[columnCount*8+256+2], &vram[columnCount*8+256], 6);
                    memmove(&vram[columnCount*8+256], &vram[columnCount*8+6], 2);
                    
                    memmove(&vram[columnCount*8+2], &vram[columnCount*8], 6);
                    
                    if (topOfScreenBlocksDown > 20) {
                        //Fill the gap at the top
                        unsigned int seed = topOfScreenBlocksDown;
                        char iii = 0;
                        for (iii= 0; iii < 28; iii+=2) {
                            seed ^= (seed << 13);
                            seed ^= (seed >> 9);
                            seed ^= (seed << 7);
                            unsigned char blockID = (seed >> 7) + TILE_BLOCK;
                            
                            SetTile(iii, 0, blockID);
                            SetTile(iii, 1, blockID);
                            SetTile(iii+1, 0, blockID);
                            SetTile(iii+1, 1, blockID);
                        }
                    }
                }
            } else {
                //We didn't wrap so just scroll up correctly
                scrollYOffset = negAmountToScroll;
            }
        }
        //////END SCROLL SCREEN

        PrintInt(3, 3, (int)GetVsyncFlag(), 0);
        WaitVsync(2);
        WaitVsync(1);
    }
    //////END MAIN GAME LOOP

    
    return 0;
}
